//GPS, Temperature, and Pressure Logger
//Captures position and temperature data and logs them to an SD card
//C. Niggel 9.27.2013

//To Do:
//strip and then try onewire again
//add comments and publish code
//maybe blink a light?check on useage of pin13
//onewire1.h - try the onewire_crc and search settings

//Application switches
//uncomment the line below to enable OneWire communications for boards with more SRAM (Leo or Mega)
#define LEO true

//uncomment the line below to enable serial output for memory use debugging
//#define MEM_DEBUG true

#include "Adafruit_GPS2.h"
#include <SoftwareSerial.h>
#include <SD.h>
#include <avr/sleep.h>
#include <Wire.h>
#include "Adafruit_MPL115A22.h"
#ifdef LEO
  #include "OneWire1.h"
  #include "DallasTemperature1.h"
#endif

//Set up the pins used by the SD card
#define chipSelect 10
#define ledPin 13

//Set up the logfile
File logfile;

//define the MPL115A2 Baro&Temp sensor
Adafruit_MPL115A2 mpl115a2;

#ifdef LEO
//If you have enough SRAM, define the OneWire sensor
//We define the hardware address manually instead of searching to save RAM
  #define ONE_WIRE_BUS 5
  OneWire oneWire(ONE_WIRE_BUS);
  DallasTemperature sensors(&oneWire);
  DeviceAddress onewireThermometer = { 0x28, 0x6F, 0x28, 0x53, 0x04, 0x0, 0x0, 0xF1 };
#endif

//Set up the GPS unit
SoftwareSerial mySerial(8, 7);
Adafruit_GPS GPS(&mySerial);
boolean usingInterrupt = false;
uint32_t timer = millis();

//Set up our variables. We make them global so they don't fragment the heap
float pressureKPA = 0, temperatureC = 0;
#ifdef LEO
  float insideTempC = 0;
#endif
//END ENVIRONMENT SETUP
//BEGIN HELPER FUNCTIONS
//Helper function for memory debugging
#ifdef MEM_DEBUG
int freeRam ()
{
    extern int __heap_start, *__brkval;
    int v;
    return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
}
#endif

//GPS interrupt code
// Interrupt is called once a millisecond, looks for any new GPS data, and stores it
SIGNAL(TIMER0_COMPA_vect) {
  char c = GPS.read();
  // if you want to debug, this is a good time to do it!
  //these two lines print to serial
  //if (true)
   // if (c) UDR0 = c;  
  // writing direct to UDR0 is much much faster than Serial.print 
  // but only one character can be written at a time.
 }

void useInterrupt(boolean v) {
  if (v) {
    // Timer0 is already used for millis() - we'll just interrupt somewhere
    // in the middle and call the "Compare A" function above
    OCR0A = 0xAF;
    TIMSK0 |= _BV(OCIE0A);
    usingInterrupt = true;
  } else {
    // do not call the interrupt function COMPA anymore
    TIMSK0 &= ~_BV(OCIE0A);
    usingInterrupt = false;
  }
}

//END HELPER FUNCTIONS
    
void setup() {
  // connect at 115200 so we can read the GPS fast enough and echo without dropping chars
  Serial.begin(115200);
  pinMode(ledPin, OUTPUT);

  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(10, OUTPUT);

#ifdef MEM_DEBUG
  Serial.println(freeRam());
#endif

  //see if the card is present and can be initialized. Spinlock if it fails.
  if (!SD.begin(chipSelect)) {
    digitalWrite(ledPin, HIGH); 
    while(1){};
  }
  char filename[15];
  strcpy(filename, "GPSLOG00.TXT");
  for (uint8_t i = 0; i < 100; i++) {
    filename[6] = '0' + i/10;
    filename[7] = '0' + i%10;
    //create if does not exist, do not open existing, write, sync after write
    if (! SD.exists(filename)) {
      break;
    }
  }
  //open up the logfile, spinlock if it fails.
  logfile = SD.open(filename, FILE_WRITE);
  if( ! logfile ) {
    digitalWrite(ledPin,HIGH);
    while(1){};
  }
  
  //enable the temp sensor
  mpl115a2.begin();
  
  #ifdef LEO 
  //enable the onewire sensor
    sensors.begin();
    sensors.getAddress(onewireThermometer,0);
    sensors.setResolution(onewireThermometer, 10);
  #endif
  
  //enable GPS
  GPS.begin(9600);
  GPS.sendCommand(PMTK_SET_NMEA_OUTPUT_RMCGGA);
  GPS.sendCommand(PMTK_SET_NMEA_UPDATE_1HZ);
  GPS.sendCommand(PGCMD_NOANTENNA);
  useInterrupt(true);

#ifdef MEM_DEBUG
   Serial.println(freeRam());
#endif
  //print logfile header
  logfile.println(F("time_utc,lat,long,altitude_M,speed_knots,kPa_out,tempC_out,tempC_in"));
}
//END SETUP

//START LOOP
void loop() {
    //read temp and pressure from the mpl sensor
    mpl115a2.getPT(&pressureKPA,&temperatureC);

    #ifdef LEO
      //read the onewire thermometer
      sensors.requestTemperatures();
      insideTempC = sensors.getTempC(onewireThermometer);
    #endif

    //these lines parse the data
    GPS.parse(GPS.lastNMEA());   // this also sets the newNMEAreceived() flag to false
    // if millis() or timer wraps around, we'll just reset it
    if (timer > millis())  timer = millis();

     // approximately every 30 seconds or so, print out the current stats
     if (millis() - timer > 30000) { 
        timer = millis(); // reset the timer
    
    char s[10];
    //write all data to logfile
    digitalWrite(ledPin,HIGH);
    logfile.print(GPS.hour);
    logfile.print(F(":"));
    if (GPS.minute < 10) logfile.print(F("0"));
    logfile.print(GPS.minute);
    logfile.print(F(":"));
    if (GPS.seconds < 10) logfile.print(F("0"));
    logfile.print(GPS.seconds);
    logfile.print(F(","));   
    char s1[10];
    logfile.print(dtostrf(GPS.latitude,5,5,s));
    logfile.print(F(","));
    logfile.print(dtostrf(GPS.longitude,5,5,s));
    logfile.print(F(","));
    logfile.print((int)GPS.altitude);
    logfile.print(F(","));
    logfile.print(dtostrf(GPS.speed,2,2,s));
    logfile.print(F(","));
    logfile.print(dtostrf(pressureKPA,2,2,s));
    logfile.print(F(","));
    logfile.print(dtostrf(temperatureC,2,2,s));
#ifdef LEO
    logfile.print(F(","));
    logfile.print(dtostrf(insideTempC,2,2,s));
#endif
    logfile.println("");
    logfile.flush();
    digitalWrite(ledPin,LOW);
    
#ifdef MEM_DEBUG
    Serial.println(freeRam());
#endif

  }
    
}


/* End code */
